function plugindef()
    finaleplugin.RequireSelection = true
    finaleplugin.Author = "Nick Mazuk"
    finaleplugin.Copyright = "©2020 Nick Mazuk"
    finaleplugin.AuthorURL = "https://nickmazuk.com"
    finaleplugin.Version = "0.1"
    finaleplugin.Date = "3/29/2020"
    return "String Harmonics 4th - Sounding Pitch", "String Harmonics 4th - Sounding Pitch", "Takes a sounding pitch, then creates the artificial harmonic that would produce that pitch"
end

function changeoctave(pitchstring, n)
    pitchstring.LuaString = pitchstring.LuaString:sub(1, -2) .. (tonumber(string.sub(pitchstring.LuaString, -1)) + n)
    return pitchstring
end

function updiatonicfourth(pitchstring)
    local letters = "ABCDEFGABCDEFG"
    local notenamepos = letters:find(pitchstring.LuaString:sub(1,1))
    local newnote = letters:sub(notenamepos + 3, notenamepos + 3)
    pitchstring.LuaString = newnote .. pitchstring.LuaString:sub(2)

    -- transposes everything an octave higher if necessary
    if(notenamepos >= 7) or notenamepos <= 2 then
        pitchstring = changeoctave(pitchstring, 1)
    end
    return pitchstring
end

function deletecirclearticulation(entry)
    local artics = entry:CreateArticulations()
        for a in eachbackwards(artics) do
            local defs = a:CreateArticulationDef()
            print(defs:GetAboveSymbolChar())
            if defs:GetAboveSymbolChar() == 111 then
                a:DeleteData()
            end
        end
end

function main() 
    for entry in eachentrysaved(finenv.Region()) do
        -- only process 1-note entries
        if (entry.Count == 1) then 
            deletecirclearticulation(entry)
            
            local note = entry:CalcLowestNote(nil)
            local pitchstring = finale.FCString()
            local measure = entry:GetMeasure()
            measureobject = finale.FCMeasure()
            measureobject:Load(measure)
            local keysig = measureobject:GetKeySignature()
            note:GetString(pitchstring, keysig, false, false)
            pitchstring = changeoctave(pitchstring, -2)
            note:SetString(pitchstring, keysig, false)
            
            local newnote = entry:AddNewNote()
            newnote.Tie = note.Tie
        
            -- makes the new note a diatonic 4th higher with the same accidentals
            newnote:SetString(updiatonicfourth(pitchstring), keysig, false)
        
            -- fixes any errors from the diatonic transposition
            if(newnote:CalcMIDIKey() - note:CalcMIDIKey() ~= 5) then
                local error = newnote:CalcMIDIKey() - note:CalcMIDIKey() - 5
                newnote.RaiseLower = newnote.RaiseLower - error
            end
        
            -- create a diamond character that is 110% of the default size
            local notehead = finale.FCNoteheadMod()
            notehead:EraseAt(newnote)
            notehead.CustomChar = 79
            notehead.Resize = 110
            if(entry:GetDuration() == 4096) then
                if(note:calcstaffposition() >= -5) then
                    notehead.horizontalpos = 5
                else
                    notehead.horizontalpos = -5
                end
            end
            notehead:SaveAt(newnote)
        end
    end
end

main()